package java.net;

import sun.misc.SharedSecrets;
import java.io.IOException;
import sun.misc.JavaIOFileDescriptorAccess;

class DualStackPlainSocketImpl extends AbstractPlainSocketImpl
{
    static JavaIOFileDescriptorAccess fdAccess;
    private final boolean exclusiveBind;
    private boolean isReuseAddress;
    static final int WOULDBLOCK = -2;
    
    public DualStackPlainSocketImpl(boolean exclusiveBind, boolean b) {
        super(b);
        this.exclusiveBind = exclusiveBind;
    }
    
    @Override
    void socketCreate(boolean b) throws IOException {
        if (this.fd == null) {
            throw new SocketException("Socket closed");
        }
        DualStackPlainSocketImpl.fdAccess.set(this.fd, socket0(b, false));
        return;
    }
    
    @Override
    void socketConnect(InetAddress inetAddress, int n, int n2) throws IOException {
        int checkAndReturnNativeFD = this.checkAndReturnNativeFD();
        if (inetAddress == null) {
            throw new NullPointerException("inet address argument is null.");
        }
        if (n2 <= 0) {
            connect0(checkAndReturnNativeFD, inetAddress, n);
        }
        else {
            configureBlocking(checkAndReturnNativeFD, false);
            try {
                if (connect0(checkAndReturnNativeFD, inetAddress, n) == -2) {
                    waitForConnect(checkAndReturnNativeFD, n2);
                }
            }
            finally {
                configureBlocking(checkAndReturnNativeFD, true);
            }
        }
        if (this.localport == 0) {
            this.localport = localPort0(checkAndReturnNativeFD);
        }
        return;
    }
    
    @Override
    void socketBind(InetAddress address, int localport) throws IOException {
        int checkAndReturnNativeFD = this.checkAndReturnNativeFD();
        if (address == null) {
            throw new NullPointerException("inet address argument is null.");
        }
        bind0(checkAndReturnNativeFD, address, localport, this.exclusiveBind);
        if (localport == 0) {
            this.localport = localPort0(checkAndReturnNativeFD);
        }
        else {
            this.localport = localport;
        }
        this.address = address;
        return;
    }
    
    @Override
    void socketListen(int n) throws IOException {
        listen0(this.checkAndReturnNativeFD(), n);
    }
    
    @Override
    void socketAccept(SocketImpl socketImpl) throws IOException {
        int checkAndReturnNativeFD = this.checkAndReturnNativeFD();
        if (socketImpl == null) {
            throw new NullPointerException("socket is null");
        }
        int n = -1;
        InetSocketAddress[] array = { null };
        if (this.timeout <= 0) {
            n = accept0(checkAndReturnNativeFD, array);
        }
        else {
            configureBlocking(checkAndReturnNativeFD, false);
            try {
                waitForNewConnection(checkAndReturnNativeFD, this.timeout);
                n = accept0(checkAndReturnNativeFD, array);
                if (n != -1) {
                    configureBlocking(n, true);
                }
            }
            finally {
                configureBlocking(checkAndReturnNativeFD, true);
            }
        }
        DualStackPlainSocketImpl.fdAccess.set(socketImpl.fd, n);
        InetSocketAddress inetSocketAddress = array[0];
        socketImpl.port = inetSocketAddress.getPort();
        socketImpl.address = inetSocketAddress.getAddress();
        socketImpl.localport = this.localport;
        return;
    }
    
    @Override
    int socketAvailable() throws IOException {
        return available0(this.checkAndReturnNativeFD());
    }
    
    @Override
    void socketClose0(boolean b) throws IOException {
        if (this.fd == null) {
            throw new SocketException("Socket closed");
        }
        if (!this.fd.valid()) {
            return;
        }
        int value = DualStackPlainSocketImpl.fdAccess.get(this.fd);
        DualStackPlainSocketImpl.fdAccess.set(this.fd, -1);
        close0(value);
        return;
    }
    
    @Override
    void socketShutdown(int n) throws IOException {
        shutdown0(this.checkAndReturnNativeFD(), n);
    }
    
    @Override
    void socketSetOption(int n, boolean isReuseAddress, Object o) throws SocketException {
        int checkAndReturnNativeFD = this.checkAndReturnNativeFD();
        if (n == 4102) {
            return;
        }
        int n2;
        switch (n) {
            case 4: {
                if (this.exclusiveBind) {
                    this.isReuseAddress = isReuseAddress;
                    return;
                }
            }
            case 1:
            case 8:
            case 4099: {
                n2 = isReuseAddress;
                break;
            }
            case 3:
            case 4097:
            case 4098: {
                n2 = (int)o;
                break;
            }
            case 128: {
                if (isReuseAddress) {
                    n2 = (int)o;
                    break;
                }
                n2 = -1;
                break;
            }
            default: {
                throw new SocketException("Option not supported");
            }
        }
        setIntOption(checkAndReturnNativeFD, n, n2);
        return;
    }
    
    @Override
    int socketGetOption(int n, Object o) throws SocketException {
        int checkAndReturnNativeFD = this.checkAndReturnNativeFD();
        if (n == 15) {
            localAddress(checkAndReturnNativeFD, (InetAddressContainer)o);
            return 0;
        }
        if (n == 4 && this.exclusiveBind) {
            return this.isReuseAddress ? 1 : -1;
        }
        int intOption = getIntOption(checkAndReturnNativeFD, n);
        switch (n) {
            case 1:
            case 4:
            case 8:
            case 4099: {
                return (intOption == 0) ? -1 : 1;
            }
            default: {
                return intOption;
            }
        }
    }
    
    @Override
    void socketSendUrgentData(int n) throws IOException {
        sendOOB(this.checkAndReturnNativeFD(), n);
    }
    
    private int checkAndReturnNativeFD() throws SocketException {
        if (this.fd == null || !this.fd.valid()) {
            throw new SocketException("Socket closed");
        }
        return DualStackPlainSocketImpl.fdAccess.get(this.fd);
    }
    
    static native void initIDs();
    
    static native int socket0(boolean p0, boolean p1) throws IOException;
    
    static native void bind0(int p0, InetAddress p1, int p2, boolean p3) throws IOException;
    
    static native int connect0(int p0, InetAddress p1, int p2) throws IOException;
    
    static native void waitForConnect(int p0, int p1) throws IOException;
    
    static native int localPort0(int p0) throws IOException;
    
    static native void localAddress(int p0, InetAddressContainer p1) throws SocketException;
    
    static native void listen0(int p0, int p1) throws IOException;
    
    static native int accept0(int p0, InetSocketAddress[] p1) throws IOException;
    
    static native void waitForNewConnection(int p0, int p1) throws IOException;
    
    static native int available0(int p0) throws IOException;
    
    static native void close0(int p0) throws IOException;
    
    static native void shutdown0(int p0, int p1) throws IOException;
    
    static native void setIntOption(int p0, int p1, int p2) throws SocketException;
    
    static native int getIntOption(int p0, int p1) throws SocketException;
    
    static native void sendOOB(int p0, int p1) throws IOException;
    
    static native void configureBlocking(int p0, boolean p1) throws IOException;
    
    static {
        DualStackPlainSocketImpl.fdAccess = SharedSecrets.getJavaIOFileDescriptorAccess();
        initIDs();
    }
}
